/**
 * Colour Banner
 */

import ColourBanner from './colour-banner.js';

const defaultBannerDefinition = require('./default-banner-definition.json');

const DEFAULT_INTRO_SLIDE_DURATION = 400; //1500;
const DEFAULT_INTRO_FADE_DURATION = 4000; //1500;
const MIN_DELTA_TIME_INTRO = 10;
const MIN_DELTA_TIME = 75;

export default class ColourBannerManager {
    constructor() {
        console.log('Create new manager');

        this.banners = [];
        this.isRunning = false;
        this.iterationDelay = 50;
        this.iterationCount = 0;
        this.parser = new DOMParser;
        this.isLoggingEnabled = false;
        this.isFirstIteration = true;
        this.firstIterationTime = 0;
        this.lastIterationTime = 0;
        this.gradientIdBase = 1;
    }

    create(parentElement, definition, id = null) {
        var banner = new ColourBanner(
            this.sanitiseDefinition(definition),
            id,
            this.parser,
            this.gradientIdBase
        );
        
        this.gradientIdBase += banner.getLayerCount() + 10;

        this.banners.push(banner);

        var bannerSvgElement = banner.element;
        var parentPosition = window.getComputedStyle(parentElement).position;
        if ((parentPosition != 'relative') && (parentPosition != 'absolute') && (parentPosition != 'fixed')) {
            parentElement.style.position = 'relative';
        }

        // console.log(`Children: ${parentElement.childNodes.length}`);
        var childElement = parentElement.firstChild;
        while (childElement != null) {
            if (childElement.nodeType == Node.ELEMENT_NODE) {
                var childPosition = window.getComputedStyle(childElement).position;
                if ((childPosition != 'relative') && (childPosition != 'absolute') && (childPosition != 'fixed')) {
                    childElement.style.position = 'relative';
                }
            }

            childElement = childElement.nextSibling;
        }

        parentElement.insertBefore(bannerSvgElement, parentElement.firstChild);

        this.start();
    }

    sanitiseDefinition(definition = {}) {
        var clonedDefaultDefinition = JSON.parse(JSON.stringify(defaultBannerDefinition));

        if (!definition) {
            definition = {};
        }
        
        definition = JSON.parse(JSON.stringify(definition));

        for (const property in clonedDefaultDefinition) {
            if (!definition.hasOwnProperty(property)) {
                definition[property] = clonedDefaultDefinition[property];
            }
        }

        if (!definition.hasOwnProperty('width')) {
            definition.width = 400;
        }

        if (!definition.hasOwnProperty('height')) {
            definition.height = 400;
        }

        if (!definition.hasOwnProperty('layers')) {
            definition.layers = [
                {}
            ];
        }

        if (!definition.hasOwnProperty('primaryColour')) {
            definition.primaryColour = '#ff00ff';
        }

        if (!definition.hasOwnProperty('slideIntro')) {
            definition.slideIntro = {};
        }

        if (!definition.slideIntro.hasOwnProperty('isEnabled')) {
            definition.slideIntro.isEnabled = true;
        }

        if (!definition.slideIntro.hasOwnProperty('duration')) {
            definition.slideIntro.duration = DEFAULT_INTRO_SLIDE_DURATION;
        }

        if (!definition.hasOwnProperty('fadeIntro')) {
            definition.fadeIntro = {};
        }

        if (!definition.fadeIntro.hasOwnProperty('isEnabled')) {
            definition.fadeIntro.isEnabled = true;
        }

        if (!definition.fadeIntro.hasOwnProperty('duration')) {
            definition.fadeIntro.duration = DEFAULT_INTRO_FADE_DURATION;
        }

        var self = this;
        definition.layers.forEach(function(layer) {
            if (!layer.phase) {
                layer.phase = 0;
            }

            if (!layer.speed) {
                layer.speed = (Math.PI * 2 * 0.05);
            }

            if (!layer.horizontalStep) {
                layer.horizontalStep = 10;
            }

            if (!layer.phaseMultiplier) {
                layer.phaseMultiplier = 1.0;
            }

            if (!layer.verticalOffset) {
                layer.verticalOffset = Math.round(definition.height * 0.95);
            }

            if (!layer.amplitude) {
                layer.amplitude = definition.height - layer.verticalOffset;
            }

            if (!layer.offsetTop) {
                layer.offsetTop = '25%';
            }

            if (!layer.opacityTop) {
                layer.opacityTop = '75%';
            }

            if (!layer.offsetBottom) {
                layer.offsetBottom = '100%';
            }

            if (!layer.opacityBottom) {
                layer.opacityBottom = '10%';
            }
        });

        return definition;
    }

    runIteration() {
        ++this.iterationCount;

        if (this.banners.length == 0) {
            this.isRunning = false;
        } else {
            var deltaTime = 0;
            var timeSinceStart = 0;
            var now = Date.now();
            var self = this;
            var bannerIndex = 0;
            var minDeltaTime = MIN_DELTA_TIME;
            var isRunningIntros = false;


            this.banners.forEach(function(banner) {
                if (banner.isIntroSlideEnabled || banner.isIntroFadeEnabled) {
                    isRunningIntros = true;
                }
            });

            if (isRunningIntros) {
                minDeltaTime = MIN_DELTA_TIME_INTRO;
            }


            if (this.lastIterationTime == 0) {
                this.firstIterationTime = now;
                this.lastIterationTime = now;
            } else {
                deltaTime = now - this.lastIterationTime;
                timeSinceStart = now - this.firstIterationTime;
            }

            if (deltaTime < minDeltaTime) {
                // ...
            } else {
                if (this.isFirstIteration) {
                    // Diagnostics
                    // console.log(deltaTime );
                    console.log(`Banners: ${this.banners.length}`);
                    console.log(`timeSinceStart: ${timeSinceStart}`);
                }

                this.banners.forEach(function(banner) {
                    if (self.isFirstIteration) {
                        // console.log(`Banner ${bannerIndex} introSlidePercentage: ${banner.introSlidePercentage}`);
                    }

                    banner.runIteration(deltaTime);

                    if (timeSinceStart <= banner.introSlideDuration) {
                        banner.introSlidePercentage = Math.sin((Math.PI / 4.0) + ((Math.PI / 4.0) * timeSinceStart / banner.introSlideDuration));
                    }

                    if (timeSinceStart <= banner.introFadeDuration) {
                        banner.introFadePercentage = Math.sin((Math.PI / 2.0) * timeSinceStart / banner.introFadeDuration);
                    }

                    ++bannerIndex;
                });

                this.isFirstIteration = false;

                this.lastIterationTime = now;
            }

            window.requestAnimationFrame(
                function() {
                    self.runIteration();
                }
            );

            // setTimeout(function() {
            //     self.runIteration();
            // }, this.iterationDelay);
        }
    }

    stop() {
        this.isRunning = false;
    }

    start() {
        if (!this.isRunning) {
            this.isRunning = true;
            this.lastIterationTime = 0;
            this.runIteration();
        }
    }
}