/**
 * Colour Banner
 */

export default class ColourBanner {
    constructor(definition, id, parser, gradientIdBase) {
        if (!definition) {
            throw 'definition cannot be null';
        }

        this.isLoggingEnabled = false;
        this.definition = JSON.parse(JSON.stringify(definition));
        this.twoPi = (Math.PI * 2.0);
        this.rootElement = null;
        this.id = null;
        this.parser = null;
        this.gradientIdBase = gradientIdBase;

        this.introSlidePercentage = 0.0;
        this.isIntroSlideEnabled = definition.slideIntro.isEnabled;
        this.introSlideDuration = definition.slideIntro.duration;

        this.introFadePercentage = 0.0;
        this.isIntroFadeEnabled = definition.fadeIntro.isEnabled;
        this.introFadeDuration = definition.fadeIntro.duration;

        if (id) {
            this.id = id;
        }

        if (parser) {
            this.parser = parser;
        }

        var self = this;
        this.definition.layers.forEach(function(layer) {
            if (!layer.colourTop) {
                layer.colourTop = self.definition.primaryColour;
            }

            if (!layer.colourBottom) {
                layer.colourBottom = self.definition.primaryColour;
            }
        });
    }

    getLayerCount() {
        return this.definition.layers.length;
    }

    getHtml() {
        var html = '';
        var styles = '';
        var defs = '';

        html += '<svg xmlns="http://www.w3.org/2000/svg" ';

        if (this.id) {
            html += `id="${this.id}" `;
        }

        html += 'class="banner-anim" ';
        html += `viewBox="0 0 ${this.definition.width} ${this.definition.height}" `;
        html += 'preserveAspectRatio="none" ';
        html += 'style="position:absolute;left:0;top:0;width:100%;height:100%;z-index:0;" ';
        html += '>';

        var pathIndex = this.gradientIdBase;
        var self = this;

        this.definition.layers.forEach(function(layer) {
            styles += '.path' + pathIndex + '{';
            styles += 'fill:' + layer.colourTop + '; ';
            styles += 'fill-opacity:' + layer.opacityBottom + '; ';
            styles += '} ';

            defs += '<linearGradient id="grad' + pathIndex + '" x1="0%" y1="0%" x2="0%" y2="100%">';
            defs += '<stop offset="' + layer.offsetTop + '" style="stop-color:' + layer.colourTop + ';stop-opacity:' + layer.opacityTop + '" />';
            defs += '<stop offset="' + layer.offsetBottom + '" style="stop-color:' + layer.colourBottom + ';stop-opacity:' + layer.opacityBottom + '" />';
            defs += '</linearGradient>';

            html += '<path ';
            html += 'id="path' + pathIndex + '" ';
            html += 'd="' + self.createCurvedLayerDef(layer) + '" ';
            // html += 'class="path' + pathIndex + '" ';
            html += 'fill="url(#grad' + pathIndex + ')" ';
            html += '/>';

            ++pathIndex;
        });

        html += '<defs>';
        html += defs;
        html += '</defs>';

        html += '</svg>';

        return html;
    }

    get element() {
        if (!this.rootElement) {
            if (this.isLoggingEnabled) {
                console.log('Creating root SVG element');
            }

            var self = this;

            this.rootElement = this.parser.parseFromString(this.getHtml(), 'image/svg+xml').firstChild;
            var pathIndex = this.gradientIdBase;
            this.definition.layers.forEach(function(layer) {
                layer.pathElement = self.rootElement.getElementById('path' + pathIndex);
                ++pathIndex;
            });
        }

        return this.rootElement;
    }

    createCurvedLayerDef(layer) {
        var x = 0;
        var atts = '';
        var scale = 1.0;
        var layerCount = this.definition.layers.length;

        if (this.isIntroSlideEnabled) {
            scale = this.introSlidePercentage;
        }

        while (x <= this.definition.width) {
            var introOffset = 0;
            var y = ((layer.verticalOffset + (layer.amplitude * Math.sin(layer.phase + (x * ((layer.phaseMultiplier * Math.PI * 2) / this.definition.width))))) * scale) - introOffset;

            // if( this.introSlidePercentage < 1.0 ) {
            //     y -= this.definition.Height * this.introSlidePercentage;
            // }

            if (atts) {
                atts += ' L'
            } else {
                atts += 'M'
            }
            atts += Math.round(x * 10.0) / 10 + ' ' + Math.round(y * 10.0) / 10;

            if (x >= this.definition.width) {
                break;
            }

            x += layer.horizontalStep;
            if (x > this.definition.width) {
                x = this.definition.width;
            }
        }

        return atts + ' L' + this.definition.width + ' 0 L0 0 Z';
    }

    runIteration(deltaTime) {
        var self = this;
        this.definition.layers.forEach(function(layer) {
            var deltaPhase = 0.0;

            if ((deltaTime > 0.0) && (layer.speed > 0.0)) {
                deltaPhase = self.twoPi * layer.speed * (deltaTime / 1000.0);
            }

            layer.phase += deltaPhase;

            while (layer.phase >= self.twoPi) {
                layer.phase -= self.twoPi;
            }

            while (layer.phase < 0.0) {
                layer.phase += self.twoPi;
            }

            // var newDefinition = self.createCurvedLayerDef(layer);
            layer.pathElement.setAttribute(
                'd',
                self.createCurvedLayerDef(layer)
            );

            // this.introFadePercentage = 0.0;
            // = true;
            // this.introFadePercentage = 0.5;
            var targetOpacity = 1.0;
            if (self.isIntroFadeEnabled) {
                targetOpacity = self.introFadePercentage; // + 0.10 ;
                if (targetOpacity >= 0.99) {
                    targetOpacity = 1.0;
                }
            }

            if ((targetOpacity >= 0.99)) {
                if (self.element.style.opacity) {
                    self.element.style.opacity = '';
                }
            } else if (self.element.style.opacity != targetOpacity) {
                self.element.style.opacity = targetOpacity;
            } else {
                // ...
            }
        });

    }

}